local window = CreateFrame("frame", "ItemProtectorOrDestroyerGui", UIParent, "BackdropTemplate")

----------------------------------------------------------------------------------------------------
-- slash command to open window
----------------------------------------------------------------------------------------------------
_G.SLASH_ITEMPROTECTORORDESTROYER1 = "/ipod"
_G.SLASH_ITEMPROTECTORORDESTROYER2 = "/itemprotector"
_G.SLASH_ITEMPROTECTORORDESTROYER3 = "/itemprotectorordestroyer"
function SlashCmdList.ITEMPROTECTORORDESTROYER() window:Show() end

----------------------------------------------------------------------------------------------------
-- window settings
----------------------------------------------------------------------------------------------------
table.insert(UISpecialFrames, window:GetName()) -- make it closable with escape key
window:SetFrameStrata("HIGH")
window:SetBackdrop({
	bgFile="Interface/Tooltips/UI-Tooltip-Background",
	edgeFile="Interface/DialogFrame/UI-DialogBox-Border",
	tile=1, tileSize=32, edgeSize=32,
	insets={left=11, right=12, top=12, bottom=11}
})
window:SetBackdropColor(0,0,0,1)
window:SetPoint("CENTER")
window:SetSize(350, 500)
window:SetMovable(true)
window:EnableMouse(true)
window:RegisterForDrag("LeftButton")
window:SetScript("OnDragStart", window.StartMoving)
window:SetScript("OnDragStop", window.StopMovingOrSizing)
window:SetScript("OnMouseDown", function(self, button)
	if button == "LeftButton" and not self.isMoving then
		self:StartMoving()
		self.isMoving = true
	end
end)
window:SetScript("OnMouseUp", function(self, button)
	if button == "LeftButton" and self.isMoving then
		self:StopMovingOrSizing()
		self.isMoving = false
	end
end)
window:SetScript("OnHide", function(self)
	if self.isMoving then
		self:StopMovingOrSizing()
		self.isMoving = false
	end
end)
window:Hide()

--------------------------------------------------
-- header title
--------------------------------------------------
local textureHeader = window:CreateTexture(nil, "ARTWORK")
textureHeader:SetTexture("Interface\\DialogFrame\\UI-DialogBox-Header")
textureHeader:SetSize(480, 64)
textureHeader:SetPoint("TOP", 0, 12)
local textHeader = window:CreateFontString(nil, "OVERLAY", "GameFontNormal")
textHeader:SetPoint("TOP", textureHeader, "TOP", 0, -14)
textHeader:SetText("Item Protector Or Destroyer 1.1.0")

--------------------------------------------------
-- checkbox options
--------------------------------------------------
local checkboxAutoTypeDelete = CreateFrame("CheckButton", "IPOD_checkboxAutoTypeDeleteDelete", window, "UICheckButtonTemplate")
checkboxAutoTypeDelete:SetPoint("TOPLEFT", window, "TOPLEFT", 16, -36)
_G[checkboxAutoTypeDelete:GetName().."Text"]:SetText('Auto-type "DELETE" on confirmations.')
checkboxAutoTypeDelete:SetScript("OnClick", function(this)
	ItemProtectorOrDestroyerSave.autoTypeDelete = this:GetChecked() or false
end)

--------------------------------------------------
-- edit box
--------------------------------------------------
local editbox = CreateFrame("Frame", "ItemProtectorOrDestroyer_editBox", window, "BackdropTemplate")
local editboxInput = CreateFrame("EditBox", "ItemProtectorOrDestroyer_editboxInput", editbox)
local editboxScroll = CreateFrame("ScrollFrame", "ItemProtectorOrDestroyer_editboxScroll", editbox, "UIPanelScrollFrameTemplate")

-- description
local textDescription = window:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
textDescription:SetJustifyH("LEFT")
textDescription:SetPoint("TOPLEFT", checkboxAutoTypeDelete, "BOTTOMLEFT", 0, -6)
textDescription:SetText("Items to protect:")

-- editbox - main container
editbox:SetPoint("TOPLEFT", textDescription, "BOTTOMLEFT", 0, -6)
editbox:SetPoint("BOTTOM", window, "BOTTOM", 0, 12)
editbox:SetWidth(window:GetRight() - window:GetLeft() - 45)
editbox:SetBackdrop({
	bgFile = "Interface/Tooltips/UI-Tooltip-Background",
	edgeFile = "Interface/DialogFrame/UI-DialogBox-Border",
	tile=1, tileSize=32, edgeSize=16,
	insets={left=5, right=5, top=5, bottom=5}})
editbox:SetBackdropColor(0,0,0,1)

-- editboxInput
editboxInput:SetMultiLine(true)
editboxInput:SetAutoFocus(false)
editboxInput:EnableMouse(true)
editboxInput:SetFont("Fonts/ARIALN.ttf", 14)
editboxInput:SetSize(editbox:GetWidth()-20, editbox:GetHeight()-8)
editboxInput:SetScript("OnEscapePressed", function() editboxInput:ClearFocus() end)
editboxInput:SetScript("OnEditFocusLost", function()
	-- save each line
	ItemProtectorOrDestroyerSave.protectedItems = {}
	for line in string.gmatch(editboxInput:GetText(), "[^\r\n]+") do
		if line:find("%w") then -- make sure it's not just spaces
			ItemProtectorOrDestroyerSave.protectedItems[line:lower()] = true
		end
	end
end)

-- editboxScroll
editboxScroll:SetPoint("TOPLEFT", editbox, "TOPLEFT", 8, -8)
editboxScroll:SetPoint("BOTTOMRIGHT", editbox, "BOTTOMRIGHT", -6, 8)
editboxScroll:EnableMouse(true)
editboxScroll:SetScript("OnMouseDown", function() editboxInput:SetFocus() end)
editboxScroll:SetScrollChild(editboxInput)

editboxInput:SetScript("OnTextChanged", function(this) ScrollingEdit_OnTextChanged(this, this:GetParent()) end)
editboxInput:SetScript("OnUpdate", function(this, elapsed) ScrollingEdit_OnUpdate(this, elapsed, this:GetParent()) end)
editboxInput:SetScript("OnCursorChanged", ScrollingEdit_OnCursorChanged)

-- dragging an item onto the editbox - has to affect the scroll box and input
local function InputReceiveItem()
	local cursor_type, _, link = GetCursorInfo()
	if cursor_type == "item" then
		local text = link:match("%[(.+)]"):lower()
		local original = editboxInput:GetText()
		if original == "" or original:sub(-1) == "\n" then
			editboxInput:SetText(original .. text)
		else
			editboxInput:SetText(original .. "\n" .. text)
		end
		editboxInput:SetFocus()
		ClearCursor()
		CloseDropDownMenus()
	end
end

editboxScroll:SetScript("OnReceiveDrag", InputReceiveItem)
local editboxScroll_OnMouseDown = editboxScroll:GetScript("OnMouseDown")
editboxScroll:SetScript("OnMouseDown", function(self, button)
	if button == "LeftButton" then InputReceiveItem() end
	editboxScroll_OnMouseDown(self, button)
end)

editboxInput:SetScript("OnReceiveDrag", InputReceiveItem)
editboxInput:SetScript("OnMouseDown", function(self, button)
	if button == "LeftButton" then InputReceiveItem() end
end)

--------------------------------------------------
-- help button
--------------------------------------------------
local buttonHelp = CreateFrame("Button", "ItemProtectorOrDestroyer_ButtonHelp", window)
buttonHelp:SetSize(16, 16)
buttonHelp:SetNormalTexture("Interface/FriendsFrame/InformationIcon")
buttonHelp:SetPoint("TOPLEFT", window, "TOPLEFT", 14, -14)
buttonHelp:SetScript("OnEnter", function()
	GameTooltip:SetOwner(window, "ANCHOR_NONE")
	GameTooltip:SetPoint("BOTTOM", window, "TOP", 0, 0)
	GameTooltip:SetText("Write each item name on a separate line.\nNames aren't case sensitive.\nYou can also drag and drop items onto the editbox.\nPress escape or close the window to save.")
	GameTooltip:Show()
end)
buttonHelp:SetScript("OnLeave", function()
	GameTooltip:Hide()
end)

--------------------------------------------------
-- close button
--------------------------------------------------
local buttonClose = CreateFrame("Button", "ItemProtectorOrDestroyer_ButtonClose", window, "UIPanelCloseButton")
buttonClose:SetPoint("TOPRIGHT", window, "TOPRIGHT", -8, -8)
buttonClose:SetScript("OnClick", function()
	editboxInput:ClearFocus()
	window:Hide()
end)

--------------------------------------------------
-- showing
--------------------------------------------------
window:SetScript("OnShow", function(self)
	checkboxAutoTypeDelete:SetChecked(ItemProtectorOrDestroyerSave.autoTypeDelete)

	-- put list in alphabetical order
	local alphabetical = {}
	local inserted
	for name in pairs(ItemProtectorOrDestroyerSave.protectedItems) do
		inserted = false
		for i=1,#alphabetical do
			if name < alphabetical[i] then
				table.insert(alphabetical, i, name)
				inserted = true
				break
			end
		end
		if not inserted then
			table.insert(alphabetical, name)
		end
	end
	if next(alphabetical) ~= nil then
		editboxInput:SetText(table.concat(alphabetical, "\n"))
	end
end)
